@@ -1,4 +1,5 @@
 utils::globalVariables(c("RMSE", "adjR", "control", "controlID",
-                         "control_coefs", "felm", "fixed", "index", "p",
+                         "control_coefs", "feols", "fixed", "index", "p",
                          "ribbon.group", "se", "sig.level", "statistic",
                          "term", "value", "."))
+
